@@ -1,5 +1,5 @@
 package Net::FreeIPA::Error;
-$Net::FreeIPA::Error::VERSION = '0.0.7';
+$Net::FreeIPA::Error::VERSION = '0.1.0';
 use strict;
 use warnings;
 
@@ -46,7 +46,9 @@ sub mkerror
 
 =item new
 
-Create new error instance from (decoded) JSON response (i.e. a hashref).
+Create new error instance from options, e.g. from a (decoded dereferenced) JSON response.
+
+Arguments are handled by C<set_error>.
 
 =cut
 
@@ -54,10 +56,78 @@ sub new
 {
     my $this = shift;
     my $class = ref($this) || $this;
-    my $self = shift || {}; # here, it gives a reference on a hash
+    my $self = {
+        __errattr => [],
+    };
     bless $self, $class;
+
+    $self->set_error(@_);
+
+    return $self;
 };
 
+=item set_error
+
+Process arguments to error
+
+=over
+
+=item no args/undef: reset the error attribute
+
+=item single argument string: convert to an C<Error> instance with message
+
+=item single argument hasref/Error instance: make a copy
+
+=item single argument/other: set Error message and save original in _orig attribute
+
+=item options (more than one arg): set the options
+
+=back
+
+=cut
+
+sub set_error
+{
+    my $self = shift;
+
+    my $nrargs = scalar @_;
+
+    my %opts;
+    if ($nrargs == 1) {
+        my $err = shift;
+        my $ref = ref($err);
+
+        if($ref eq 'Net::FreeIPA::Error') {
+            %opts = map {$_ => $err->{$_}} @{$err->{__errattr}};
+        } elsif ($ref eq 'HASH') {
+            %opts = %$err;
+        } elsif (defined($err) && $ref eq '') {
+            $opts{message} = $err;
+        } elsif (defined($err)) {
+            $opts{message} = "unknown error type $ref, see _orig attribute";
+            $opts{_orig} = $err;
+        }
+    } elsif ($nrargs > 1) {
+       %opts = @_;
+    }
+
+
+    # Wipe current state
+    # Do this after the %opts are build, to allow copies of itself
+    foreach my $key (@{$self->{__errattr}}) {
+        delete $self->{$key};
+    }
+    $self->{__errattr} = [];
+
+    # sort produces sorted __errattr
+    foreach my $key (sort keys %opts) {
+        $self->{$key} = $opts{$key};
+        push(@{$self->{__errattr}}, $key);
+    }
+
+    return $self;
+}
+
 =item is_error
 
 Test if this is an error or not.
